@@ -45,7 +45,7 @@
 //!     // other members ...
 //! }
 //! impl From<u64> for PageTableEntry { /* ... */ }
-//! impl Into<u64> for PageTableEntry { /* ... */ }
+//! impl From<PageTableEntry> for u64 { /* ... */ }
 //! ```
 //!
 //! This generated bitfield then can be used as follows.
@@ -66,6 +66,7 @@
 //! ```
 
 use proc_macro as pc;
+use proc_macro2::Span;
 use proc_macro2::TokenStream;
 use quote::{format_ident, quote, ToTokens};
 use syn::parse::{Parse, ParseStream};
@@ -116,7 +117,7 @@ fn bitfield_inner(args: TokenStream, input: TokenStream) -> syn::Result<TokenStr
         #vis struct #name(#ty);
 
         impl #name {
-            #vis fn new() -> Self {
+            #vis const fn new() -> Self {
                 Self(0)
             }
 
@@ -128,9 +129,9 @@ fn bitfield_inner(args: TokenStream, input: TokenStream) -> syn::Result<TokenStr
                 Self(v)
             }
         }
-        impl Into<#ty> for #name {
-            fn into(self) -> #ty {
-                self.0
+        impl From<#name> for #ty {
+            fn from(v: #name) -> #ty {
+                v.0
             }
         }
     })
@@ -140,24 +141,21 @@ fn bitfield_member(f: syn::Field, pty: &Type, offset: &mut usize) -> syn::Result
     let ty = &f.ty;
 
     let mut bits = type_bits(ty)?;
-    match attr_bits(&f.attrs)? {
-        Some(b) => {
-            if b > bits {
-                return Err(syn::Error::new_spanned(&f, "member type not large enough"));
-            }
-            if b == 0 {
-                return Err(syn::Error::new_spanned(&f, "bits may not be 0"));
-            }
-            bits = b;
+    if let Some(b) = attr_bits(&f.attrs)? {
+        if b > bits {
+            return Err(syn::Error::new_spanned(&f, "member type not large enough"));
         }
-        _ => {}
+        if b == 0 {
+            return Err(syn::Error::new_spanned(&f, "bits may not be 0"));
+        }
+        bits = b;
     }
 
     let doc: TokenStream = f
         .attrs
         .iter()
         .filter(|a| a.path.is_ident("doc"))
-        .map(|a| a.to_token_stream())
+        .map(ToTokens::to_token_stream)
         .collect();
 
     let start = *offset;
@@ -178,37 +176,35 @@ fn bitfield_member(f: syn::Field, pty: &Type, offset: &mut usize) -> syn::Result
     let vis = &f.vis;
 
     if bits > 1 {
+        let mask: u128 = !0 >> (u128::BITS as usize - bits);
+        let mask = LitInt::new(&format!("0x{:x}", mask), Span::mixed_site());
         Ok(quote! {
             #doc
-            #vis fn #with_name(mut self, value: #ty) -> Self {
-                self.#set_name(value);
-                self
+            #vis const fn #with_name(mut self, value: #ty) -> Self {
+                Self(self.0 & !(#mask << #start) | (value as #pty & #mask) << #start)
             }
             #doc
-            #vis fn #name(&self) -> #ty {
+            #vis const fn #name(&self) -> #ty {
                 (((self.0 >> #start) as #ty) << #ty::BITS as usize - #bits) >> #ty::BITS as usize - #bits
             }
             #doc
             #vis fn #set_name(&mut self, value: #ty) {
-                self.0 &= !(((1 << #bits) - 1) << #start);
-                self.0 |= (value as #pty & ((1 << #bits) - 1)) << #start;
+                *self = self.#with_name(value);
             }
         })
     } else {
         Ok(quote! {
             #doc
-            #vis fn #with_name(mut self, value: #ty) -> Self {
-                self.#set_name(value);
-                self
+            #vis const fn #with_name(mut self, value: #ty) -> Self {
+                Self(self.0 & !(1 << #start) | (value as #pty & 1) << #start)
             }
             #doc
-            #vis fn #name(&self) -> #ty {
+            #vis const fn #name(&self) -> #ty {
                 ((self.0 >> #start) & 1) != 0
             }
             #doc
             #vis fn #set_name(&mut self, value: #ty) {
-                self.0 &= !(1 << #start);
-                self.0 |= (value as #pty & 1) << #start;
+                *self = self.#with_name(value);
             }
         })
     }
